/* demonstrate a bug
 * call first a prepared statement : we have a correct result
 * use this prepare statement plenty of times with other parameters and we call it again with the initial value
 * we have a other result (wrong)
 * 
 
OS : Linux fedora core 3 =Linux 2.6.10-1.770_FC3 #1 Thu Feb 24 14:00:06 EST 2005 i686 athlon i386 GNU/Linux
JAVA VM = sun 1.5.0_01
My sql version = 4.1.13-standard
connector/J = 3.1.10

 */
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.StringTokenizer;


/**
 * @author christian
 *
 */
public class DBconnectionTest extends com.mysql.jdbc.util.BaseBugReport {
	
	static final String traceFile="trace.txt";
	transient private PreparedStatement prepStatementSearchMatchesWithValue;
	// read the pattern for the sql select request (select ... where ... LIKE 'pattern')
	private void readFromFile(ArrayList<Item> listOfValueForTesting) throws IOException
	{
		BufferedReader fic=new BufferedReader(new FileReader(traceFile));
		String str;
		do
		{
			str=fic.readLine();
			if (str!=null)
			{
				StringTokenizer st=new StringTokenizer(str,",",true);
				String pattern=(String)st.nextElement();
				st.nextElement(); //skip the ,
				String length=(String)st.nextElement();
				listOfValueForTesting.add(new Item(pattern,(new Integer(length)).intValue()));
			}
		}
		while (str!=null);
		fic.close();
	}
	
	private void cleanUp() throws SQLException
	{
		//delete temporary data, as the used words and 
		// close database connection
		
		if (prepStatementSearchMatchesWithValue!=null)
		{
			prepStatementSearchMatchesWithValue.close();
		}
		
	}
	private void buildPreparedStatements() throws SQLException
	{					
		prepStatementSearchMatchesWithValue=getConnection().prepareStatement("SELECT word_id,formatted FROM words WHERE def_level_1_available AND unuse_until <CURRENT_DATE AND length(formatted)<= ? AND formatted LIKE ?");
		
	}
	public void testMySqlConnector()
	{
	}
	private int testSearch(String pattern,int sizeWord) throws SQLException
	{
		ResultSet rs;
		int nbItems=0;
		prepStatementSearchMatchesWithValue.setString(2,pattern);
		prepStatementSearchMatchesWithValue.setInt(1,sizeWord);
		
		rs =prepStatementSearchMatchesWithValue.executeQuery();
		while (rs.next()) 
		{
			nbItems++;
		}
		rs.close();
		return nbItems;
	}
	
	public void runTest() throws Exception {
		ArrayList<Item> listOfValueForTesting=new ArrayList<Item>();
		readFromFile(listOfValueForTesting);
		int max=listOfValueForTesting.size();
		
		// get the number of match for a SQL select
		Item errorElement =  listOfValueForTesting.get(max-1);
		int resultBefore=testSearch(errorElement.getStr(),errorElement.getSize());	// expect 0
		System.out.println("first select gives "+resultBefore+" matches");
		
		// run until last-1
		int i=0;
		for (Iterator iter = listOfValueForTesting.iterator(); i<max-1;i++)
		{
			Item element = (Item) iter.next();
			testSearch(element.getStr(),element.getSize());
		}
		
		//make the same request as for the first time
		int resultAfter=testSearch(errorElement.getStr(),errorElement.getSize());	//got 34 !!!
		System.out.println("same select later gives "+resultAfter+" matches");
		assertTrue("2 different result set for the same SQL request",resultAfter==resultBefore);		
	}
	public void setUp() throws Exception {
		createTable();
		buildPreparedStatements();
		
	}
	/**
	 * @throws SQLException 
	 * 
	 */
	private void createTable() throws SQLException {
		
		Statement stmt=getConnection().createStatement();
		stmt.executeUpdate("CREATE TABLE words (\n"+
				"word_id MEDIUMINT UNSIGNED NOT NULL,\n"+
				"def_level_1_available BOOL NOT NULL,\n"+
				"def_level_2_available BOOL NOT NULL,\n"+
				"def_level_3_available BOOL NOT NULL,\n"+
				"formatted VARCHAR(20) NOT NULL,\n"+
				"unformatted VARCHAR(20) NOT NULL,\n"+
				"unuse_until DATE NOT NULL,\n"+
				"PRIMARY KEY (word_id))"
		);
		
		stmt.executeUpdate("CREATE UNIQUE INDEX word_formatted_1_ndx ON words(def_level_1_available,unuse_until,formatted)");
		stmt.executeUpdate("CREATE UNIQUE INDEX word_formatted_2_ndx ON words(def_level_2_available,unuse_until,formatted)");
		stmt.executeUpdate("CREATE UNIQUE INDEX word_formatted_3_ndx ON words(def_level_3_available,unuse_until,formatted)");
		
		stmt.executeUpdate("CREATE INDEX formatted_word_level_1_ndx ON words(def_level_1_available,unuse_until,formatted(8))");
		stmt.executeUpdate("CREATE INDEX formatted_word_level_2_ndx ON words(def_level_2_available,unuse_until,formatted(8))");
		stmt.executeUpdate("CREATE INDEX formatted_word_level_3_ndx ON words(def_level_3_available,unuse_until,formatted(8))");
		
		// fill the table
		stmt.executeUpdate("LOAD DATA LOCAL INFILE \"data_bd.txt\" INTO TABLE words");
		stmt.close();
	}
	
	public void tearDown() throws Exception {
		cleanUp();
		Statement stmt=getConnection().createStatement();
		stmt.executeUpdate("DROP TABLE words");
		stmt.close();
		getConnection().close();
		
	}
	
	public static void main(String[] args) throws Exception {
		new DBconnectionTest().run();
	}
}
